<?php
/**
 * @copyright   Copyright (C) Chatbro. All rights reserved.
 * @license     GNU General Public License version 2 or later.
 */

defined('_JEXEC') or die();
defined('CHATBROENGINE') || die();

use ChatbroChatbroappCoreCbro_backendable as CbroBackendable;
use ChatbroChatbroappAdminCbro_admin as CbroAdmin;
use ChatbroChatbroappUtilsCbro_utils as CbroUtils;
use ChatbroChatbroappSettingsCbro_setting as CbroSetting;
use ChatbroChatbroappSettingsCbro_input_type as CbroInputType;
use ChatbroChatbroappSettingsCbro_settings_iterator as CbroSettingsIterator;
use ChatbroChatbroappExceptionsCbro_setting_not_found as CbroSettingNotFound;

class ChatbroChatbroappSettingsCbro_settings extends CbroBackendable
{
	const guid = 'chatbro_chat_guid';
	const secret = 'chatbro_chat_secret';
	const display = 'chatbro_chat_display';
	const selected_pages = 'chatbro_chat_selected_pages';
	const user_profile_path = 'chatbro_chat_user_profile_url';
	const enabled = 'chatbro_enabled';
	const enable_shortcodes = 'chatbro_enable_shortcodes';
	const plugin_version = 'chatbro_plugin_version';

	public $settings;

	protected function __construct($backend)
	{
		parent::__construct($backend);

		$this->settings = [];
		$backend->reg_store();

		$this->add_setting(
			new CbroSetting($backend, [
				'id' => self::enabled,
				'type' => CbroInputType::checkbox,
				'label' => 'Plugin enabled',
				'sanitizer' => [
					$backend->get_cbro_utils_class(),
					'sanitize_checkbox',
				],
				'default' => true,
				'active' => true,
			])
		);

		$this->add_setting(
			new CbroSetting($backend, [
				'id' => self::guid,
				'type' => CbroInputType::text,
				'label' => 'Account ID',
				'sanitizer' => [
					$backend->get_cbro_utils_class(),
					'sanitize_guid',
				],
				'generator' => [$backend->get_cbro_utils_class(), 'gen_guid'],
				'required' => true,
				'pattern' =>
					"[\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12}$",
				'pattern_error' => 'Invalid chat key',
				'active' => true,
			])
		);

		$this->add_setting(
			new CbroSetting($backend, [
				'id' => self::secret,
				'type' => CbroInputType::text,
				'label' => 'Account secret',
				'sanitizer' => [
					$backend->get_cbro_utils_class(),
					'sanitize_secret',
				],
				'generator' => [$backend->get_cbro_utils_class(), 'gen_secret'],
				'required' => true,
				'active' => true,
			])
		);

		$this->add_setting(
			new CbroSetting($backend, [
				'id' => self::user_profile_path,
				'type' => CbroInputType::text,
				'label' => 'User profile path',
				'default' => CbroUtils::get_default_profile_url(),
				'addon' => CbroUtils::get_home_url(),
				'required' => false,
				'active' => false,
			])
		);

		if (CbroAdmin::has_shortcodes()) {
			$this->add_setting(
				new CbroSetting($backend, [
					'id' => self::enable_shortcodes,
					'type' => CbroInputType::checkbox,
					'label' => 'Shortcodes enabled',
					'sanitizer' => [
						$backend->get_cbro_utils_class(),
						'sanitize_checkbox',
					],
					'default' => true,
					'active' => true,
				])
			);
		}

		$backend->add_extra_settings($this);
	}

	public function add_setting($setting)
	{
		$this->settings[$setting->id()] = $setting;
	}

	public static function get_setting($id)
	{
		if (!array_key_exists($id, self::get_instance()->settings)) {
			throw new CbroSettingNotFound($id);
		}

		return self::get_instance()->settings[$id];
	}

	public static function get($id)
	{
		return self::get_setting($id)->get();
	}

	public static function set($id, $value)
	{
		self::get_setting($id)->set($value);
	}

	public static function set_sanitized($id, $value)
	{
		self::get_setting($id)->set_sanitized($value);
	}

	public static function iterator()
	{
		return new CbroSettingsIterator(
			array_keys(self::get_instance()->settings)
		);
	}

	public static function del($id)
	{
		self::get_setting($id)->del();
	}
}
